==> qcs_sdk_python-0.12.9.tar.gz: qcs_sdk_python-0.12.9/crates/python/src/qpu/result_data.rs <==
use std::collections::HashMap;

use pyo3::{
    pyclass, pymethods,
    types::{PyComplex, PyFloat, PyInt, PyList},
    IntoPy, Py, PyResult, Python,
};
use qcs::qpu::{QpuResultData, ReadoutValues};
use rigetti_pyo3::{py_wrap_type, py_wrap_union_enum, PyTryFrom, PyWrapper, ToPython};

py_wrap_union_enum! {
    PyReadoutValues(ReadoutValues) as "ReadoutValues" {
        integer: Integer => Vec<Py<PyInt>>,
        real: Real => Vec<Py<PyFloat>>,
        complex: Complex => Vec<Py<PyComplex>>
    }
}

py_wrap_type! {
    PyQpuResultData(QpuResultData) as "QPUResultData"
}

#[pymethods]
impl PyQpuResultData {
    #[new]
    fn __new__(
        py: Python<'_>,
        mappings: HashMap<String, String>,
        readout_values: HashMap<String, PyReadoutValues>,
    ) -> PyResult<Self> {
        Ok(Self(QpuResultData::from_mappings_and_values(
            mappings,
            HashMap::<String, ReadoutValues>::py_try_from(py, &readout_values)?,
        )))
    }

    #[getter]
    fn mappings(&self, py: Python<'_>) -> PyResult<HashMap<String, String>> {
        self.as_inner().mappings().to_python(py)
    }

    #[getter]
    fn readout_values(&self, py: Python<'_>) -> PyResult<HashMap<String, PyReadoutValues>> {
        self.as_inner().readout_values().to_python(py)
    }

    pub(crate) fn to_raw_readout_data(&self, py: Python<'_>) -> RawQpuReadoutData {
        RawQpuReadoutData {
            mappings: self.as_inner().mappings().clone(),
            readout_values: self
                .as_inner()
                .readout_values()
                .iter()
                .map(|(register, values)| {
                    (
                        register.to_string(),
                        match values {
                            ReadoutValues::Integer(values) => PyList::new(py, values).into_py(py),
                            ReadoutValues::Real(values) => PyList::new(py, values).into_py(py),
                            ReadoutValues::Complex(values) => PyList::new(py, values).into_py(py),
                        },
                    )
                })
                .collect::<HashMap<String, Py<PyList>>>(),
        }
    }
}

/// A wrapper type for data returned by the QPU in a more flat structure than
/// [`PyQpuResultData`] offers. This makes it more convenient to work with
/// the data if you don't care what type of number the readout values for
/// each register contains.
#[derive(Debug)]
#[pyclass(name = "RawQPUReadoutData")]
pub struct RawQpuReadoutData {
    #[pyo3(get)]
    mappings: HashMap<String, String>,
    #[pyo3(get)]
    readout_values: HashMap<String, Py<PyList>>,
}

impl RawQpuReadoutData {
    fn __repr__(&self) -> String {
        format!("{self:?}")
    }
}

==> qcs_sdk_python-0.13.5rc0.tar.gz: qcs_sdk_python-0.13.5rc0/crates/python/src/qpu/result_data.rs <==
use std::collections::HashMap;

use pyo3::{
    pyclass, pymethods,
    types::{PyComplex, PyFloat, PyInt, PyList},
    IntoPy, Py, PyResult, Python,
};
use qcs::qpu::{result_data::MemoryValues, QpuResultData, ReadoutValues};
use rigetti_pyo3::{impl_repr, py_wrap_type, py_wrap_union_enum, PyTryFrom, PyWrapper, ToPython};

py_wrap_union_enum! {
    PyReadoutValues(ReadoutValues) as "ReadoutValues" {
        integer: Integer => Vec<Py<PyInt>>,
        real: Real => Vec<Py<PyFloat>>,
        complex: Complex => Vec<Py<PyComplex>>
    }
}

py_wrap_union_enum! {
    PyMemoryValues(MemoryValues) as "MemoryValues" {
        binary: Binary => Vec<Py<PyInt>>,
        integer: Integer => Vec<Py<PyInt>>,
        real: Real => Vec<Py<PyFloat>>
    }
}

py_wrap_type! {
    PyQpuResultData(QpuResultData) as "QPUResultData"
}
impl_repr!(PyQpuResultData);

#[pymethods]
impl PyQpuResultData {
    #[new]
    fn __new__(
        py: Python<'_>,
        mappings: HashMap<String, String>,
        readout_values: HashMap<String, PyReadoutValues>,
        memory_values: HashMap<String, Option<PyMemoryValues>>,
    ) -> PyResult<Self> {
        Ok(Self(QpuResultData::from_mappings_and_values(
            mappings,
            HashMap::<String, ReadoutValues>::py_try_from(py, &readout_values)?,
            HashMap::<String, Option<MemoryValues>>::py_try_from(py, &memory_values)?,
        )))
    }

    #[getter]
    fn mappings(&self, py: Python<'_>) -> PyResult<HashMap<String, String>> {
        self.as_inner().mappings().to_python(py)
    }

    #[getter]
    fn readout_values(&self, py: Python<'_>) -> PyResult<HashMap<String, PyReadoutValues>> {
        self.as_inner().readout_values().to_python(py)
    }

    #[getter]
    fn memory_values(&self, py: Python<'_>) -> PyResult<HashMap<String, Option<PyMemoryValues>>> {
        self.as_inner().memory_values().to_python(py)
    }

    pub(crate) fn to_raw_readout_data(&self, py: Python<'_>) -> RawQpuReadoutData {
        RawQpuReadoutData {
            mappings: self.as_inner().mappings().clone(),
            readout_values: self
                .as_inner()
                .readout_values()
                .iter()
                .map(|(register, values)| {
                    (
                        register.to_string(),
                        match values {
                            ReadoutValues::Integer(values) => PyList::new(py, values).into_py(py),
                            ReadoutValues::Real(values) => PyList::new(py, values).into_py(py),
                            ReadoutValues::Complex(values) => PyList::new(py, values).into_py(py),
                        },
                    )
                })
                .collect::<HashMap<String, Py<PyList>>>(),
            memory_values: self
                .as_inner()
                .memory_values()
                .iter()
                .map(|(register, values)| {
                    (
                        register.to_string(),
                        match values {
                            Some(MemoryValues::Binary(values)) => {
                                Some(PyList::new(py, values).into_py(py))
                            }
                            Some(MemoryValues::Integer(values)) => {
                                Some(PyList::new(py, values).into_py(py))
                            }
                            Some(MemoryValues::Real(values)) => {
                                Some(PyList::new(py, values).into_py(py))
                            }
                            None => None,
                        },
                    )
                })
                .collect::<HashMap<String, Option<Py<PyList>>>>(),
        }
    }
}

/// A wrapper type for data returned by the QPU in a more flat structure than
/// [`PyQpuResultData`] offers. This makes it more convenient to work with
/// the data if you don't care what type of number the readout values for
/// each register contains.
#[derive(Debug)]
#[pyclass(name = "RawQPUReadoutData")]
#[pyo3(get_all)]
pub struct RawQpuReadoutData {
    mappings: HashMap<String, String>,
    readout_values: HashMap<String, Py<PyList>>,
    memory_values: HashMap<String, Option<Py<PyList>>>,
}

impl RawQpuReadoutData {
    fn __repr__(&self) -> String {
        format!("{self:?}")
    }
}
